\section{Etude Generation}

One use of the performer model is in the generation of simplistic yet useful etudes, customized to match the abilities of a given performer.  The {\tt EtudeGeneration} module includes a set of functions for generating such etudes.

\begin{verbatim} 

> module EtudeGeneration where
> import Ratio
> import List
> import Basics
> import ComposersWorkbench
> import Miscellany
> import Scores
> import SaxophoneFingerings
> import Performer

\end{verbatim}

\subsection{Selecting Transitions to Test}

  The etudes that are generated will be simple sequences of two- or three-note melodies, each of which includes a transition which it would be beneficial for the performer to exercise.  The tempo of the etude and the rhythms of its melodies are chosen such that each transition is played at or near the performer's maximum speed.  

\begin{verbatim}

>  -- Fingering Transition
> type FingeringTrans = (SaxophoneFingering, SaxophoneFingering)
> type TempoRange = (Integer, Integer)
> -- Transition with Tempo Ranges
> type TransitionWTR = (FingeringTrans, TempoRange, [(TempoRange, Dur)])

\end{verbatim}

  To determine the transitions that will appear in the etude, a function is provided which takes a musical phrase and finds all of the fingering transitions in the performer database that are either at a lower speed in the database than they are in the phrase, or are of a low confidence in the database.  Since each transition will become a two- or three-note phrase in the etude, selecting low confidence transitions will allow these transitions to be tested in isolation.  The selection of transitions of lower speed in the database than in the phrase is perhaps obvious, since these are precisely the transitions that the performer needs to exercise to be able to play the phrase correctly.

\begin{verbatim}

> transitionsToTest :: TechniqueMap -> Music -> Integer -> Integer -> 
>                      [FingeringTrans] 
> transitionsToTest t m bpm bpMeas = map f (filter isTestable fctm)
>   where f :: (SaxFCombination, FingeringTempo) -> FingeringTrans
>         f ((a, b), _) = (lookupFingering a, lookupFingering b)
>         isTestable :: (SaxFCombination, FingeringTempo) -> Bool
>         isTestable (fc, mt) 
>           = (confidence t fc) <= lowConfidence || 
>             mt > (maximumTempo t fc) 
>         ft :: [FingeringTempo]
>         ft = fingeringTempos m bpm bpMeas
>         fctm :: [(SaxFCombination, FingeringTempo)]
>         fctm = zip (fingeringCombinations m) ft

\end{verbatim}

  The {\tt generateEtude} function takes this list of transitions and then generates an etude.  This is a multi-step process that is described in depth in the remainder of this section.

\begin{verbatim}

> generateEtude :: TechniqueMap -> [FingeringTrans] -> Title -> Score
> generateEtude tm ts title = Score title (4%4) t (Key C Major) m
>   where t = selectTempo trs -- tempo
>         (trs,twtrs') = findTempoRI twtrs -- the origional twtrs
>         twtrs = (genRanges.sortTrans.(trainingRanges tm)) ts 
>         m = sequencePhrases mels -- music 
>         -- melody fragments, derived from each transition
>         mels :: [Music] 
>         mels = map (\(ft, tr, tdm) -> altMel t ft tdm) twtrs'
>         altMel :: Integer -> FingeringTrans -> [(TempoRange, Dur)] -> 
>                   Music -- alternatig Melody fragment generator
>         altMel tempo ft tdm 
>           = alternatingMelody (findPitchSeq ft) (selectDur tempo tdm)         

\end{verbatim}

\subsection{Generating a Tempo and Note Durations}

  First, the the maximum tempo for each transition is retrieved from the performer database, and a tempo range is created to indicate the range of preferred speeds a which each transition should be tested, in the etude.  These transitions and associated tempo ranges are then ordered from the lowest to the highest range.  This ordering is used in future steps to assure that the best mapping from tempo ranges to durations of notes is used.

\begin{verbatim}

> trainingRanges :: TechniqueMap -> [FingeringTrans] -> 
>                   [(FingeringTrans, TempoRange)] 
> trainingRanges tm ts = map f ts
>   where f t = let a = (maximumTempo tm (transToCombo t)) 
>               in (t, (a, a+10))

> transToCombo :: FingeringTrans -> SaxFCombination
> transToCombo (a,b) = (fingeringNumber a, fingeringNumber b)

> sortTrans :: [(FingeringTrans, TempoRange)] -> 
>              [(FingeringTrans, TempoRange)]
> sortTrans = sortBy (\(a,b) (c,d) -> compare b d)

\end{verbatim}

  Next, a set of tempo ranges for various note duration types is generated.  Since, for example, a sixteenth note at 40 bpm has the same real duration as a quarter note at 160 bpm, it is useful to generate, for each transition, a tempo range for each note type that might be used in the etude.  Later, a particular set of note durations will be chosen from these sets of ranges which will allow one tempo to be used for all of the different parts of the etude.

\begin{verbatim}

> genRanges :: [(FingeringTrans, TempoRange)] -> [TransitionWTR]
> genRanges = map genTWTR
>   where genTWTR :: (FingeringTrans, TempoRange) -> TransitionWTR
>         genTWTR (s, t@(t1, t2)) = (s, t, tempoRangeMap t)

> tempoRangeMap :: TempoRange -> [(TempoRange, Dur)]
> tempoRangeMap (t0,t1) = filter isModerate allTs
>   where daf :: [(Dur, Ratio Int)]  -- Durs and Factors 
>         daf = [(1%32, 1%2), (1%24, 2%3), (1%16, 1), (1%12, 4%3),
>                (1%8, 2), (1%6, 8%3), (1%4, 4)]
>         allTs :: [(TempoRange, Dur)]
>         allTs = map f daf
>         f :: (Dur, Ratio Int) -> (TempoRange, Dur)
>         f (a,b) = ((g t0 b, g t1 b), a)
>         g :: Integer -> Ratio Int -> Integer
>         g x y = (iRatioFloor.ratioIntToRational) ((fromInteger x) * y)
>         isModerate ((a,b),c) = (a >= 30) && (b <= 170)

\end{verbatim}

  The next step is to search for a tempo to use for the etude by finding the interstection of all of the tempo ranges generated by the previous step.  As required, the lower bound of a given tempo range may be lowered to make it match the others, but the upper bound will never be raised.  This assures that the difficulty of the piece will never be too great for the performer.

\begin{verbatim}

> -- Find Tempo Range Intersection
> --   The input must be sorted for this function to work properly
> findTempoRI :: [TransitionWTR] -> ([TempoRange], [TransitionWTR]) 
> findTempoRI [] = error "findTempoRI: list cannot be empty"
> findTempoRI ((t@(_,_,tr)):ts) = findTempoRIAcc ts ((map fst tr),[t])

> findTempoRIAcc :: [TransitionWTR] -> ([TempoRange], [TransitionWTR]) -> 
>                   ([TempoRange], [TransitionWTR])
> findTempoRIAcc [] x = x
> findTempoRIAcc ((t@((s1,s2),(tempo0,tempo1),trd)):ts) a@(tr',t')
>   = if (null x) 
>     then findTempoRIAcc (((s1,s2),y,z):ts) a
>     else findTempoRIAcc ts (x, t'++[t])
>     where x = intersectTR (map fst trd) tr'
>           y = (tempo0 - 5,tempo1)
>           z :: [(TempoRange, Dur)]
>           z = tempoRangeMap y 

> intersectTR :: [TempoRange] -> [TempoRange] -> [TempoRange]
> intersectTR [] _ = []
> intersectTR _ [] = []
> intersectTR (r:rs) rs1 = (limitTR r rs1) ++ (intersectTR rs rs1)

> limitTR :: TempoRange -> [TempoRange] -> [TempoRange]
> limitTR _       [] = []
> limitTR (t0,t1) ((t2,t3):rs) 
>   = if t2 <= t1 && t3 >= t0 && t1 >= t0
>     then (a,b):(limitTR (t0,t1) rs)
>     else limitTR (t0,t1) rs
>     where a = if t0 <= t2 then t2 else t0
>           b = if t3 >= t1 then t1 else t3

\end{verbatim}

  A tempo is then selected that is in the middle of the range of acceptable tempos determined previously.  For each transition, the largest note duration whose tempo range includes the selected tempo for the etude is selected for use in one of the two- or three-note melodies in the etude.

\begin{verbatim}

> selectTempo :: [TempoRange] -> Integer
> selectTempo rs = floor $ (fromInteger (mr0 + mr1)) / 2
>   where (mr0,mr1) = median rs

> median :: (Ord a) => [a] -> a
> median [] = error "can't find the median of an empty set"
> median x  = x' !! (floor ((fromInt (length x')) / 2))
>   where x' = sort x 

> selectDur :: Integer -> [(TempoRange, Dur)] -> Dur
> selectDur t m = maximum ds
>   where ds = [d | (_,d) <- filter (inRange t) m]
>         inRange t' ((t0,t1),_) = t' >= t0 && t' <= t1

\end{verbatim}

\subsection{Generating Pitches and Assembling the Etude}

  Once the durations have been selected for each two- or three-note melody, the pitch content for each melody is determined.  This is a little trickier than it might appear at first, since one would like to select a sequence of pitches for which the "correct" fingering sequence includes the transition to be tested.  In most cases, it is adequate to select a pitch sequence by simply mapping the fingerings in the sequence to the pitches they produce.  In the case where the derived melody does not include the desired transition, however, one must modify that melody so that the transition will occur naturally.  The easiest way to do this is to replace all occurrences of the pitch $x$, whose fingering ($f_{x1}$) is not the desired fingering ($f_{x2}$) with a sequence of pitches $<x, y, x>$ where the transition $<f_{x2},f_y>$ has the highest value of all transitions from $f_{x2}$ to some $f_y$.  

  Consider, for example, the fingering sequence  $<f_{fork}, f_a>$ where $f_{fork}$ is the fingering for the awkward, but sometimes useful "fork key" fingering for F\# in the key of Eb, and $f_a$ is the fingering for A in the key of Eb.  The correct fingering for the melody $<F\#, A>$ is $<f_{m}, f_a>$, where $f_{m}$ is the ``middle finger'' fingering for F\#.  Clearly, the trivial mapping of fingerings to pitches is not adequate in this case.  The correct fingering for $<F\#, F, F\#, A>$ , however, is $<f_{fork}, f_f, f_{fork}, f_a>$, where $f_f$ is the fingering for F-natural.  This melody does include the desired transition, $<f_{fork}, f_a>$.  In effect, the insertion of the pitch F, ``forces'' the ``relatively awkward'' fingering sequence $<f_{fork}, f_a>$, for which we are trying to generate a melody.

\begin{verbatim}

> findPitchSeq :: FingeringTrans -> [Pitch]
> findPitchSeq (s1,s2) = (f s1 s1p) ++ (f s2 s2p)
>   where initSeq = findFingeringSeq [s1p,s2p]
>         s1p     = fingeringToPitch s1
>         s2p     = fingeringToPitch s2
>         f a b   = if (a `elem` initSeq) 
>                   then [b] 
>                   else [b,lowestCostP a,b]

> lowestCostP :: SaxophoneFingering -> Pitch
> lowestCostP s = fingeringToPitch s'
>   where s' = maximumBy f allFingerings 
>         allFingerings = map lookupFingering [1..48]
>         f :: SaxophoneFingering -> SaxophoneFingering -> Ordering
>         f s0 s1 = compare (g s0) (g s1)
>         g s' = saxFValue [s,s']

\end{verbatim}

  Now, given a set of melodic fragments that we would like to appear in the etude, all that remains is to arrange them together into one piece of music.  Perhaps the simplest way to do this is to make one phrase for each fragment, ending each with a held tone.  The {\tt alternatingMelody} function is used to assemble the pitches and durations for each transition into one of these phrases.  Among other things, the function assures that the generated phrase will begin and end on the downbeat.

\begin{verbatim}

> alternatingMelody :: [Pitch] -> Dur -> Music
> alternatingMelody p d = makeMusic p' dSeq
>   where rp  = concat (repeat p)
>         isTriplet = (rd' `mod` 3) == 0
>         rd' = if (isWholeNum rd) 
>              then iRatioFloor rd 
>              else error "input duration is not a supported note length"
>         rd  = recip d
>         np' = if isTriplet then 13 else 9
>         p'  = take np' rp
>         na  = (length p') - 1
>         minLastD  = (fromInt na) * d
>         minTotalD = 2 * minLastD
>         lastD :: Dur
>         lastD = if isWholeNum minTotalD 
>                 then minLastD
>                 else (fromInt (ceiling (fromIRatio minTotalD)))
>                      - minLastD
>         dSeq  = (take na (repeat d)) ++ [lastD]
>         makeMusic :: [Pitch] -> [Dur] -> Music
>         makeMusic ps ds = foldl1 (:+:) (zipWith (\p d -> (Note p d [])) ps ds)

\end{verbatim}

This procedure for generating an etude produces functional but aesthetically uninteresting etudes.  Similar techniques to those shown here might be employed to rearrange the melodic frangments to make the result more interesting, melodically and rhythmically.


