
\section{Fingering Selection}

  A saxophone fingering is a set of keys that are depressed on the saxophone to produce a pitch.  Each fingering produces only one pitch, but several fingerings may produce a common pitch.  In common practice, fingering selection is left up to the performer, and it is expected that he will choose the best sequence of fingerings for any given phrase.  For a performer some fingering sequences will be much more difficult than others, so the difficulty of a piece depends both on the sequences of fingerings that are selected and the performer's ability to execute those sequences at a given tempo.

  The {\tt SaxophoneFingerings} module serves as a means to generate a ``correct'' fingering sequence for a given musical phrase.  The ``correct'' fingering sequence is here defined as the easiest sequence for an intermediate to advanced saxophonist to execute, according to the author's experience as a saxophonist.  The design of the module is sufficiently general, however, that it could be modified to fit the tastes of other saxophonists, or even to be dynamically adjusted to match the abilities of a given perforer as their technique improves.

\begin{verbatim}

> module SaxophoneFingerings where
> import Ix
> import Ratio
> import Graph
> import Basics
> import ComposersWorkbench

\end{verbatim}

\subsection{Fingering Representation}

  The basis for this representation of saxophone fingerings is the key layout of the Selmer Mark VI Alto Saxophone.  Each fingering is given a unique number to simplify identification and indexing.  Fingerings are represented with this number, a list of keys that are simultaneously depressed, and the (concert) pitch that will be produced as a result.  

\begin{verbatim}

> type SaxophoneFingeringNumber = Integer

> data SaxophoneFingering 
>   = SaxFingering SaxophoneFingeringNumber 
>                  [LeftHandKey] [RightHandKey] Pitch
>     deriving (Read, Show, Eq)    

Abbreviations:

Symbol Meaning
------ -------
LH     Left Home  
LP     Left Pinkey (used to play low notes)
LS     Left Side   (used to play high Eb, D, and F in the key of Eb)
T      Thumb
u      Upper

Examples:

Symbol  Used to play this pitch, in Eb
------  ------------------------------
LH1     B
LH1l    Bb
LP1     G#
LP2     low B 
LP3     low C#
LP4     low Bb
LS1     high D
LS2     high Eb
LS3     high F

> data LeftHandKey = LH1u | LH1 | LH1l | LH2 | LH3 
>                  | LP1 | LP2 | LP3 | LP4 | LS1 | LS2 | LS3 | T
>     deriving (Read, Show, Eq)    

Abbreviations:

Symbol Meaning
------ -------
RH     Right Home  
RP     Right Pinkey (used to play low notes)
RS     Right Side   (used to play middle Bb, C, etc.)
u      Upper

Examples:

Symbol  Used to play this pitch, in Eb
------  ------------------------------
RH1     F
RH3u    F#
RP1     D#
RP2     low C 
RS1     high E
RS2     middle C
RS3     middle Bb

> data RightHandKey 
>   = RH1 | RH2 | RH3 | RH3u | RP1 | RP2 | RS1 | RS2 | RS3
>     deriving (Read, Show, Eq)    

\end{verbatim}

  A weighted graph of {\tt SaxophoneFingering}s is used in the selection algorithm.  Since the graph data type used requires its elements to be indexable and ordered, it is necessary to make {\tt SaxophoneFingering} an instance of these classes.   

\begin{verbatim}

> instance Ord SaxophoneFingering where
>   compare (SaxFingering n _ _ _) (SaxFingering m _ _ _) = compare n m

> instance Ix SaxophoneFingering where
>   range (SaxFingering n _ _ _, SaxFingering m _ _ _) 
>     = map lookupFingering (range (n,m))
>   index (SaxFingering n _ _ _, SaxFingering m _ _ _) 
>         (SaxFingering x _ _ _) = index (n,m) x
>   inRange (SaxFingering n _ _ _, SaxFingering m _ _ _) 
>           (SaxFingering x _ _ _) = inRange (n,m) x

\end{verbatim}

  Notably, this representation is not sufficient to express extended techniques like half-hole fingerings and multiphonics.  Likewise it is possible to use this representation to make nonsense fingerings.  Nonetheless, it is a realtively simple and intuitive representation that is sufficient to express the most common fingerings on the saxophone. Additionally, while not all saxophones have the same key chart, the chart used here is one of the most common.

  Given a suitable representation, the next step is to encode all of the saxophone fingerings in that representation.  The variable names used indicate the register and pitch produced, in the key of Eb, by each fingering.  For example the fingering which produces the concert pitch C one octave above middle C is given the name highAF. This corresponds to the name saxophonists commonly use for this fingering -- ``the fingering for high A''.

\begin{verbatim}

> lowBbF   = sf 1  (lh123 ++ [LP4]) rhLowC           (Cs,3)
> lowBF    = sf 2  (lh123 ++ [LP2]) rhLowC           (D, 3)
> lowCF    = sf 3  lh123            rhLowC           (Ds,3)
> lowCsF   = sf 4  (lh123 ++ [LP3]) rhLowC           (E, 3)
> lowDF    = sf 5  lh123            rh123            (F, 3)
> lowDsF   = sf 6  lh123            (rh123 ++ [RP1]) (Fs,3)
> lowEF    = sf 7  lh123            rh12             (G, 3)
> lowFF    = sf 8  lh123            [RH1]            (Gs,3)
> -- low F# middle finger
> lowFs1F  = sf 9  lh123            [RH2]            (A, 3) 
> -- low F# trill key
> lowFs2F  = sf 10 lh123            [RH1, RH3u]      (A, 3) 
> midGF    = sf 11 lh123            []               (As,3) 
> midGsF   = sf 12 (lh123 ++ [LP1]) []               (B, 3) 
> midAF    = sf 13 lh12             []               (C, 4) 
> -- middle A# sidekey
> midAs1F  = sf 14 lh12             [RS3]            (Cs,4) 
> -- middle A# index finders
> midAs2F  = sf 15 [LH1]            [RH1]            (Cs,4) 
> -- middle A# middle finger
> midAs3F  = sf 16 [LH1]            [RH2]            (Cs,4) 
> -- middle A# two keys under left index
> midAs4F  = sf 17 [LH1, LH1l]      []               (Cs,4) 
> midBF    = sf 18 [LH1]            []               (D, 4) 
> -- middle C middle finger
> midC1F   = sf 19 [LH2]            []               (Ds,4) 
> -- middle C side key
> midC2F   = sf 20 [LH1]            [RS2]            (Ds,4) 
> -- middle C overtone
> midC3F   = addT 21 lowCF                            
> -- middle C# open
> midCs1F  = sf 22 []               []               (E, 4) 
> -- middle C# overtone
> midCs2F  = addT 23 lowCsF
> midDF    = addT 24 lowDF
> midDsF   = addT 25 lowDsF
> midEF    = addT 26 lowEF
> midFF    = addT 27 lowFF
> midFs1F  = addT 28 lowFs1F
> midFs2F  = addT 29 lowFs2F
> highGF   = addT 30 midGF
> highGsF  = addT 31 midGsF
> highAF   = addT 32 midAF
> highAs1F = addT 33 midAs1F
> highAs2F = addT 34 midAs2F
> highAs3F = addT 35 midAs3F
> highAs4F = addT 36 midAs4F
> highBF   = addT 37 midBF
> highC1F  = addT 38 midC1F
> highC2F  = addT 39 midC2F
> highCsF  = addT 40 midCs1F
> highDF   = sf 41 [T, LS1]            []           (F, 5)
> highDsF  = sf 42 [T, LS1, LS2]       []           (Fs,5)
> highE1F  = sf 43 [T, LS1, LS2]       [RS1]        (G, 5)
> highE2F  = sf 44 [T, LH1u, LH2, LH3] []           (G, 5)
> highE3F  = sf 45 [T, LS3]            []           (G, 5)
> highE4F  = sf 46 [T, LS1, LH1u]      []           (G, 5)
> highF1F  = sf 47 [T, LS1, LS2, LS3]  [RS1]        (Gs,5)
> highF2F  = sf 48 [T, LH1u, LH2]      []           (Gs,5)

> sf = SaxFingering
> rhLowC = rh123 ++ [RP2] -- right hand of low C
> rh123  = rh12 ++ [RH3]  -- right hand home 123
> rh12   = [RH1, RH2]     -- right hand home 12
> lh123  = lh12 ++ [LH3]  -- left hand home 123
> lh12   = [LH1, LH2]     -- left hand home 12
> addT n' (SaxFingering n lh rh (pc,o)) 
>   = (SaxFingering n' (T:lh) rh (pc,o+1))

\end{verbatim}

For this representation, it is straightforward to deine functions that map from pitches to fingerings and from fingerings to pitches.

\begin{verbatim}

> saxophoneFingerings :: Pitch -> [SaxophoneFingering]
> saxophoneFingerings p = saxF p'
>   where p' :: Pitch
>         p' = trans (-3) p -- alto sax Eb = concert c

> saxF :: Pitch -> [SaxophoneFingering]
> saxF p@(pc, o) = case (pc, o) of
>   -- enharmonics
>   (Bf, _) -> saxF (As, o);    (Cf, _) -> saxF (B, (o-1))
>   (Bs, _) -> saxF (C, (o+1)); (Df, _) -> saxF (Cs, o)
>   (Ef, _) -> saxF (Ds, o);    (Ff, _) -> saxF (E, o)
>   (Es, _) -> saxF (F, o);     (Gf, _) -> saxF (Fs, o)
>   (Af, _) -> saxF (Gs, o)
>   -- low range
>   (As, 2) -> [lowBbF]; (B,  2) -> [lowBF]; (C,  3) -> [lowCF] 
>   (Cs, 3) -> [lowCsF]; (D,  3) -> [lowDF]; (Ds, 3) -> [lowDsF]
>   (E,  3) -> [lowEF];  (F,  3) -> [lowFF]; 
>   (Fs, 3) -> [lowFs1F, lowFs2F]
>   -- middle range
>   (G,  3) -> [midGF];  (Gs, 3) -> [midGsF]; (A,  3) -> [midAF]
>   (As, 3) -> [midAs1F, midAs2F, midAs3F, midAs4F]
>   (B,  3) -> [midBF];  
>   (C,  4) -> [midC1F, midC2F, midC3F]
>   (Cs, 4) -> [midCs1F, midCs2F]      
>   (D,  4) -> [midDF];  (Ds, 4) -> [midDsF]; (E,  4) -> [midEF]                 
>   (F,  4) -> [midFF];  
>   (Fs, 4) -> [midFs1F, midFs2F]   
>   -- high range
>   (G,  4) -> [highGF]; (Gs, 4) -> [highGsF]; (A,  4) -> [highAF]                  
>   (As, 4) -> [highAs1F, highAs2F, highAs3F, highAs4F] 
>   (B,  4) -> [highBF]
>   (C,  5) -> [highC1F, highC2F] 
>   (Cs, 5) -> [highCsF]; (D,  5) -> [highDF]; (Ds, 5) -> [highDsF]  
>   (E,  5) -> [highE1F, highE2F, highE3F, highE4F] 
>   (F,  5) -> [highF1F, highF2F] 
>   -- altissimo (none yet)
>   _      -> error ("saxF: no fingerings for this pitch (in Eb): " 
>                    ++ (show p))

> fingeringToPitch :: SaxophoneFingering -> Pitch
> fingeringToPitch (SaxFingering _ _ _ p) = p

\end{verbatim}

For indexing, it is also useful to have a set of functions for mapping fingerings to their unique identifiers and back.

\begin{verbatim}

> lookupFingering :: SaxophoneFingeringNumber -> SaxophoneFingering
> lookupFingering n = case n of
>  1  -> lowBbF;   2  -> lowBF;    3  -> lowCF;    4  -> lowCsF
>  5  -> lowDF;    6  -> lowDsF;   7  -> lowEF;    8  -> lowFF
>  9  -> lowFs1F;  10 -> lowFs2F;  11 -> midGF;    12 -> midGsF
>  13 -> midAF;    14 -> midAs1F;  15 -> midAs2F;  16 -> midAs3F
>  17 -> midAs4F;  18 -> midBF;    19 -> midC1F;   20 -> midC2F
>  21 -> midC3F;   22 -> midCs1F;  23 -> midCs2F;  24 -> midDF
>  25 -> midDsF;   26 -> midEF;    27 -> midFF;    28 -> midFs1F
>  29 -> midFs2F;  30 -> highGF;   31 -> highGsF;  32 -> highAF
>  33 -> highAs1F; 34 -> highAs2F; 35 -> highAs3F; 36 -> highAs4F
>  37 -> highBF;   38 -> highC1F;  39 -> highC2F;  40 -> highCsF
>  41 -> highDF;   42 -> highDsF;  43 -> highE1F;  44 -> highE2F
>  45 -> highE3F;  46 -> highE4F;  47 -> highF1F;  48 -> highF2F

> fingeringNumber :: SaxophoneFingering -> SaxophoneFingeringNumber
> fingeringNumber (SaxFingering n _ _ _) = n 

\end{verbatim}

Finally, a function is provided to determine if two fingerings sound the same.  

\begin{verbatim}

> soundsSame :: SaxophoneFingering -> SaxophoneFingering -> Bool
> (SaxFingering n _ _ _) `soundsSame` (SaxFingering n1 _ _ _) 
>   = if n == n1 then True else b
>   where g = [[9,10], [14..17], [19..21], [22,23], [28,29], [33..36],
>              [38,39], [43..46], [47,48]]
>         b = or (map (\x -> (elem n x) && (elem n1 x)) g)

\end{verbatim}

To demonstrate this representation, the following program prints to the console the entire saxophone fingering chart.

\begin{verbatim}

> printSaxFingeringChart :: IO ()
> printSaxFingeringChart = putStr saxFingeringChart

> saxFingeringChart :: String
> saxFingeringChart = "\nSaxophone Fingering Chart\n\n" ++ diagrams
>   where diagrams       = concat (map showDiagrams allSaxPitches)
>         showDiagrams x = (heading x) ++ (fingeringDiagrams x) 
>                          ++ separator
>         heading      x = (show (trans (-3) x)) ++ ":" 
>         separator      = "\n------------------------" ++ 
>                          "----------------------\n"
>         -- all pitches in the (basic) saxophone range concert c#3 - Ab5
>         allSaxPitches :: [Pitch]
>         allSaxPitches = map pitch [(1+12*3)..(8+12*5)]

> fingeringDiagrams :: Pitch -> String
> fingeringDiagrams p 
>   = concat (map (\d -> "\n" ++ (fingeringDiagram d)) ds)
>   where ds = saxophoneFingerings p

> fingeringDiagram :: SaxophoneFingering -> String
> fingeringDiagram (SaxFingering n lh rh _)  = show lh ++ " " ++ show rh

\end{verbatim}

\subsection{A Weighted Graph of Saxophone Fingerings}
 
The algorithm for selecting a fingering sequence given a piece of music uses a weighted graph of saxophone fingering transitions, where the weight is an estimate of the difficulty of the transition.  The higher the weight, the easier it is to play, or the more rapidly it may be played with ease.  Given a sequence of pitches \begin{math}<p_0, ..., p_n>\end{math} the graph is used to find the sequence of fingerings \begin{math}<f_0, ..., f_n>\end{math} with the greatest weight $w$ where $w$ is the sum of all pairs \begin{math}(f_x, f_{x+1})\end{math}.

To construct this graph, the author charted the maximum tempos at which he could play all of the transitions for which there are multiple transitions that produce the same pitch sequence.  A default weight was used for all intervals for which there is only one fingering combination.  Although somewhat time-consuming, this approach is comprehensive and produces very satisfactory results.  Other techniques for populating this graph might include a rule-based approach, or the use of physical models to simulate difficulty in technique.  For example one might collect a body of rules like ``jumping octaves is costly'', or ``never use the F\# fork key unless the adjoining pitch is F''.  Alternatively, one might encode the number and positions of fingers employed in each fingering, and then give a higher weight to transitions that cause the performer's fingers to move the least.

  The type synonym {\tt FingeringTempo} is used to encode the graph weights.  It is a measure of the rapidity with which one can play a given fingering transition.  Specifically, it represents the tempo (quarter-note beats per minute in 4/4) at which the author can play a given fingering sequence in 16th-notes.  Unfortunately, it was not possible to display the weighted graph neatly when printing this paper.  It is included here in part, and the source code is availble upon request. 

\begin{verbatim}

> type FingeringTempo = Integer 

> saxFGraph :: Graph SaxophoneFingering FingeringTempo
> saxFGraph = mkGraph False (lowBbF, highF2F) 
>             ((mw lowBbF [(2, 50), (3, 112), (4, 54), (5, 44), (6, 26), (7, 52), (8, 52), (9, 60), (10, 50), (11, 30), (12, 54), (13, 30), (14, 27), (15, 33), (16, 33), (17, 38), (18, 33), (19, 42), (20, 44), (21, 36), (22, 60), (23, 30), (24, n), (25, n), (26, n), (27, n), (28, 54), (29, 40), (30, n), (31, n), (32, n), (33, 40), (34, 35), (35, 35), (36, 46), (37, n), (38, 56), (39, 60), (40, n), (41, n), (42, n), (43, 17), (44, 14), (45, 23), (46, 17), (47, 23), (48, 25)]) ++ 
>              (mw lowBF ((ns 3 8)  ++ (f948 48  44  44 56  58  60  56  66 42 66  27 58 48 60 52 60 54 52 66 40 17 38 28 36 44))) ++
>              (mw lowCF  ((ns 4 8) ++ (f948 60  38  76 84  84  92  84  69 66 76  66 69 50 72 63 66 66 52 69 36 27 42 36 33 44))) ++
>              (mw lowCsF ((ns 5 8) ++ (f948 112 48  80 72  63  84  50  46 30 48  44 44 30 40 46 42 52 44 34 38 24 34 20 30 33))) ++ 
>              (mw lowDF  ((ns 6 8) ++ (f948 120 48  72 84  72  88  72  60 50 40  30 56 30 50 54 52 66 50 40 40 26 34 22 30 33))) ++
>              (mw lowDsF ((ns 7 8) ++ (f948 138 48  66 92  92  100 66  54 52 50  30 63 33 50 60 58 63 54 42 44 28 36 24 30 33))) ++
>              (mw lowEF  ((8, n):     (f948 144 84  76 116 104 126 76  54 46 100 36 66 44 60 66 60 70 68 72 46 28 38 24 33 36))) ++
>              (mw lowFF               (f948 108 168 80 138 46  130 100 70 56 100 50 58 60 56 63 50 58 70 74 50 28 40 24 38 34)) ++
>              (mw lowFs1F (zip [11..48] [168,160,168,96,72,168,140,108,126,70,100,132,60,56,54,56,40,66,24,160,160,132,63,58,66,60,132,56,63,104,104,104,50,30,36,28,48,36])) ++ 
>              (mw lowFs2F (zip [11..48] [144,140,140,50,126,40,116,100,126,66,30, 120,30,28,28,30,60,40,72,100,100,100,60,66,54,60,100,56,66,90, 90, 90, 44,28,36,30,40,28])) ++ 
>              (mw midGF  ((ns 12 13) ++ (f1448 96  50 48 144 116 80 100 144 90  84  76  54 56 56 58 53 66 46 30 38 32 44 30))) ++
>              (mw midGsF ((13, n):      (f1448 96  50 48 136 112 76 80  140 100 80  70  54 56 56 58 53 66 46 30 38 32 44 30))) ++
>              (mw midAF                 (f1448 192 60 58 168 160 92 84  160 80  108 104 80 54 54 66 50 66 46 22 20 22 44 50)) ++
>              (mw midAs1F (zip [18..48] [144,116,30, 44, 100,50,52,52,52,52,66, 68, 120,120,160,88,40,40,40,130,48,46,86, 86, 86, 50,44,40,34,60,50])) ++
>              (mw midAs2F (zip [18..48] [168,60, 100,100,94, 66,76,76,76,76,60, 104,90, 90, 60, 40,88,40,40,160,24,60,94, 94, 94, 84,52,64,42,68,58])) ++
>              (mw midAs3F (zip [18..48] [160,58, 80, 90, 90, 68,74,74,74,50,120,60, 90, 90, 60, 40,40,88,40,156,20,58,90, 90, 90, 82,50,62,40,70,60])) ++
>              (mw midAs4F (zip [18..48] [50, 63, 24, 104,144,70,68,68,68,68,110,100,140,140,140,40,40,40,88,80, 40,40,116,116,116,80,40,60,30,80,50])) ++
>              (mw midBF   (f1948 50 150 54 160 60 100 90 120 170 160 60 46 100 60 40 50 38 50 34)) ++
>              (mw midC1F   (zip [22..48] [160,60,72,72,72,72,120,110,126,126,126,90,46,44,36, 60, 108,40, 120,114,110,56,20,50,20,56,30])) ++
>              (mw midC2F   (zip [22..48] [120,56,50,50,50,50,80, 70, 90, 90, 90, 46,60,58,30, 100,40, 100,80, 70, 70, 30,20,28,20,20,20])) ++
>              (mw midC3F  (zip [22..48] [60,130,64,60,64,64,70, 50, 60, 60, 60, 36,40,38,42, 58, 40, 40, 70, 60, 60, 20,20,20,20,20,20])) ++
>              (mw midCs1F  (zip [24..48]        [76,76,80,80,100,96, 126,126,126,90,80,78,100,98, 66, 50, 120,120,120,42,28,30,32,34,30])) ++
>              (mw midCs2F (zip [24..48]        [74,60,74,74,80, 70, 60, 60, 60, 70,60,58,80, 56, 40, 30, 70, 70, 70, 24,20,20,22,22,20])) ++
>              (mw midDF  ((ns 25 27) ++ (f2848 100 30  52 76 74 68 72 50 50 30 40 28 44 38))) ++
>              (mw midDsF ((ns 26 27) ++ (f2848 100 30  52 76 74 68 72 50 50 30 40 28 44 38))) ++
>              (mw midEF  ((27, n):      (f2848 110 30  52 76 74 68 72 50 50 30 40 28 44 38))) ++
>              (mw midFF                 (f2848 50  110 52 76 74 68 72 50 50 30 40 28 44 38)) ++
>              (mw midFs1F (zip [30..48] [160,160,132,80,70,152,148,132,100,70,120,120,120,50,30,36,28,48,36])) ++
>              (mw midFs2F (zip [30..48] [100,100,100,76,140,60,120,100,90, 60,100,100,100,40,20,26,20,46,34])) ++
>              (mw highGF ((ns 31 32) ++ (f3348 120 90 90 140 130 94 50 30 40 28 90 40))) ++
>              (mw highGsF ((32, n):     (f3348 120 90 90 140 130 94 50 30 40 28 90 40))) ++
>              (mw highAF                (f3348 160 60 60 140 130 94 50 30 40 28 90 40)) ++
>              (mw highAs1F (zip [37..48] [130,104,50,110,90, 90, 30,20,28,20,30,20])) ++
>              (mw highAs2F (zip [37..48] [160,60, 64,100,80, 80, 50,20,40,20,48,20])) ++
>              (mw highAs3F (zip [37..48] [156,58, 60,100,80, 80, 48,20,38,20,46,20])) ++
>              (mw highAs4F (zip [37..48] [80, 64, 40,152,100,100,46,20,36,20,50,20])) ++
>              (mw highBF   (f3848 80 140 50 20 46 20 50 40)) ++
>              (mw highC1F (zip [40..48] [152,100,90,60,80,40,40,60,80])) ++
>              (mw highC2F (zip [40..48] [110,80, 70,50,30,42,30,50,30])) ++
>              (mw highCsF ((ns 41 42) ++ (f4348 110 100 120 90 100 90))) ++
>              (mw highDF  ((42, n) :     (f4348 108 70  80  70 90  70))) ++
>              (mw highDsF                (f4348 110 70  90  70 90  70)) ++
>              (mw highE1F                (f4748                138 52)) ++
>              (mw highE2F                (f4748                50  120)) ++
>              (mw highE3F                (f4748                100 40)) ++
>              (mw highE4F                (f4748                50  40)))
>  where n = 10 -- default weight
>        ns l h = [(x,n) | x <- [l..h]]
>        mw fingering weights = [(fingering, lookupFingering f, w) | (f,w) <- weights] -- map weights
>        f4748 w47 w48 = [(47, w47), (48, w48)]
>        f4348 w43 w44 w45 w46 w47 w48 
>          = [(43, w43), (44, w44), (45, w45), (46, w46)] ++ (f4748 w47 w48)
>        f3848 w38 w39 w43 w44 w45 w46 w47 w48 
>          = [(38, w38), (39, w39)] ++ (ns 40 42) ++ (f4348 w43 w44 w45 w46 w47 w48)
>        f3348 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48 
>          = [(33, w33), (34, w34), (35, w35), (36, w36), (37, n)] ++
>            (f3848 w38 w39 w43 w44 w45 w46 w47 w48) 
>        f2848 w28 w29 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48 
>          = [(28, w28), (29, w29)] ++ (ns 30 32) ++ 
>            (f3348 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48)
>        f1948 w19 w20 w21 w22 w23 w28 w29 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48 
>          = [(19, w19), (20, w20), (21, w21), (22, w22), (23, w23)] ++ 
>            (ns 24 27) ++ (f2848 w28 w29 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48) 
>        f1448 w14 w15 w16 w17 w19 w20 w21 w22 w23 w28 w29 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48 
>          = [(14, w14), (15, w15), (16, w16), (17, w17), (18, n)] ++ 
>            (f1948 w19 w20 w21 w22 w23 w28 w29 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48)
>        f948 w9 w10 w14 w15 w16 w17 w19 w20 w21 w22 w23 w28 w29 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48 
>          = [(9, w9), (10, w10)] ++ (ns 11 13) ++ 
>            (f1448 w14 w15 w16 w17 w19 w20 w21 w22 w23 w28 w29 w33 w34 w35 w36 w38 w39 w43 w44 w45 w46 w47 w48)

\end{verbatim}

  The current implementation of the search algorithm is the trivial one -- all of the possible fingering sequences are enumerated, and the highest value path is selected.  This limits the usefulness of this function to short phrases.  An algorithm that prunes the graph iteratively would increase the performance of this function.

\begin{verbatim}

> findFingeringSequence :: Music -> [SaxophoneFingering]
> findFingeringSequence m = findFingeringSeq (extractPitches m)

> findFingeringSeq :: [Pitch] -> [SaxophoneFingering]
> findFingeringSeq pitches 
>   = findHWPath saxFValue (map (\p -> (saxophoneFingerings p)) pitches) 

> findAllFingerings :: Music -> [[SaxophoneFingering]]
> findAllFingerings m = ffs (extractPitches m) []
>   where extendS f s' = map (\s -> s++[f]) s'
>         ffs :: [Pitch] -> [[SaxophoneFingering]] -> 
>                [[SaxophoneFingering]] 
>         ffs []     s  = s
>         ffs (p:ps) [] 
>           = ffs ps (map (\f -> [f]) (saxophoneFingerings p))
>         ffs (p:ps) s' 
>           = ffs ps (concat (map (\f -> extendS f s') 
>                                 (saxophoneFingerings p)))

> saxFValue :: [SaxophoneFingering] -> Integer
> saxFValue []       = 0
> saxFValue (x:[])   = 0
> saxFValue (a:b:xs) = if (a == b) 
>                      then restC 
>                      else if (a `soundsSame` b) 
>                           then -100 
>                           else ((weight a b saxFGraph) + restC)
>   where restC = saxFValue (b:xs)

\end{verbatim}






